# -*- coding: utf-8 -*-
###
# this program is under the GPL v3 license :
# http://www.gnu.org/licenses/gpl-3.0.html
#
# you are totally free to install, use, modify
# but if you distribute it, you have to distribute also the code...  (that's in GPL)
###
from sys import exit, argv
from time import sleep, time
from cv2 import COLOR_RGB2GRAY, cvtColor, VideoCapture

from PyQt4.QtGui import *
from PyQt4.QtCore import *
import numpy as np

import matplotlib
from scipy.ndimage.filters import gaussian_filter1d

from ui_webcamProfile import Ui_webcamProfile
from camera import CameraDevice, CameraWidget

class DropDownCamera(QComboBox):
    def promote(self,cameraDevice):
        self.cameraDevice = cameraDevice

    def mousePressEvent(self, QMouseEvent):
        cam_range = range(1,5)
        for i in cam_range[::-1]:
            self.removeItem(i)
        self.setCursor(Qt.WaitCursor)
        self.cameraDevice.paused = True
        for i in cam_range:
            sleep(1)
            cam = VideoCapture(i)
            if cam.isOpened():
                self.addItem("cam "+str(i))
            else:
                self.addItem("cam "+str(i)+" not available")
                self.model().item(i).setEnabled(False)
            cam.release()
        self.setCursor(Qt.ArrowCursor)
        QComboBox.mousePressEvent(self, QMouseEvent)


class RecordButton(QPushButton):
    @property
    def nb2show(self):
        return self.remaining_frames

    @nb2show.setter
    def nb2show(self,i):
        self.remaining_frames = i

    def promote(self):
        self.pen0= QPen(QColor(0,0,0),1)
        self.penOn= QPen(QColor(250,0,0),20,Qt.SolidLine, Qt.RoundCap, Qt.BevelJoin)
        self.penOff= QPen(QColor(150,150,150),20,Qt.SolidLine, Qt.RoundCap, Qt.BevelJoin)
        self.textOption=QTextOption(Qt.Alignment(Qt.AlignBottom + Qt.AlignRight))
        self.contour = QRectF(0,0,self.width()-1,self.height()-1)
        self.remaining_frames = 10

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setPen(self.penOn if self.isChecked() else self.penOff)
        painter.drawPoint(self.width()/2,self.height()/2)
        painter.setPen(self.pen0)
        painter.drawText(QRectF(0,0,self.width(),self.height()),str(self.remaining_frames),self.textOption)
        painter.drawRect(self.contour)


class listWrap(list):
    def __getitem__(self, y):
        return super(listWrap, self).__getitem__(np.mod(y,len(self)))

#    def __getitem__(self, item):

class PointTB(QPoint):
    def distanceTo(self, p3):
        diff = self - p3
        return np.hypot(diff.x(), diff.y())


class LigneTB(QLine):

    colorList=listWrap((QColor(200, 200, 0, 150), QColor(0, 200, 200, 150), QColor(200, 0, 200, 150)))
    styleList= listWrap((Qt.SolidLine, Qt.DashLine, Qt.DashDotLine, Qt.DotLine)) # ,Qt.DashDotDotLine, Qt.CustomDashLine))
    spinBoxSize=QSize(30, 20)
    spinBoxCenter=QPoint(15, 10)

    def __init__(self, p1, p2, i1=None, i2=None, parent=None, etendue=20, color=None, nb=0):
        if color == None:
            color=LigneTB.colorList[nb]
        self.lineStyle = LigneTB.styleList[nb]
        self.color = color.toRgb()
        self.color.setAlpha(100)
        self.color2 = color
        self.color2.setAlpha(40)
        self.parent = parent
        self.etendue = etendue
        if isinstance(p1, QPoint) & isinstance(p2, QPoint):
            super(LigneTB, self).__init__(p1, p2)
        # elif isinstance(p1,int) & isinstance(p2,int) & isinstance(i1,int) & isinstance(i2,int) :
        else:
            super(LigneTB, self).__init__(p1, p2, i1, i2)
        self.lengthEdit = QSpinBox(parent)
        self.lengthEdit.setGeometry(QRect(self.center() - LigneTB.spinBoxCenter, LigneTB.spinBoxSize))
        self.lengthEdit.setStyleSheet("QSpinBox{background:transparent;border:1px dotted rgb" +
                                      str(self.color.getRgb()[0:3]) + """}
                                      QSpinBox::up-button,QSpinBox::down-button {width:16px;left:4px}""")
        self.lengthEdit.setMaximum(np.hypot(self.parent.width(), self.parent.height()))
        self.lengthEdit.setValue(self.length())
        self.lengthEdit.editingFinished.connect(self.on_lengthEdit_validated)
        # self.lengthEdit.__class__ = EditLenght
        self.etendueEdit = QSpinBox(parent)
        self.etendueEdit.setGeometry(QRect(self.etenduePos() - LigneTB.spinBoxCenter, LigneTB.spinBoxSize))
        self.etendueEdit.setProperty("value", self.etendue)
        self.etendueEdit.setStyleSheet(self.lengthEdit.styleSheet())
        self.etendueEdit.setMaximum(parent.height())
        self.etendueEdit.editingFinished.connect(self.on_etendueEdit_validated)
        self.p1InMove= self.p2InMove = self.inMove = False
        self.stretchable = True

    def p1(self):
        return PointTB(QLine.p1(self))
    def p2(self):
        return PointTB(QLine.p2(self))

    def center(self):
        return (self.p1() + self.p2()) / 2.

    def etenduePos(self):
        if self.length()>80:
            self.etendueEdit.hide()
        return 20./self.length() * self.p1()  +  (1 - 20./self.length()) * self.p2()

    def length(self):
        diff = self.p1() - self.p2()
        return np.hypot(diff.x(), diff.y())

    def theta(self):
        diff = self.p1() - self.p2()
        if diff.x() == 0:
            return np.pi / 2
        else:
            return 1. * np.arctan(1. * diff.y() / diff.x())

    def future_p(self, orig,hand):
        diff= orig-hand
        future=QPointF()
        if diff.x() == 0:
            future.setX(orig.x())
            future.setY(orig.y() + 1.*np.sign(diff.y()*self.length()))
        else:
            theta=np.arctan(1. * diff.y() / diff.x())
            future.setX(orig.x() - 1. * np.sign(diff.x()) * self.length() * np.cos(theta))
            future.setY(orig.y() - 1. * np.sign(diff.x()) * self.length() * np.sin(theta) )
        return future.toPoint()

    @property
    def stretchable(self):
        """
        boolean True if user can stretch the line by pulling line's ending.
        False if length is fixed.
        :return: Bool "stretchable"
        """
        return self.lengthEdit.isEnabled()

    @stretchable.setter
    def stretchable(self, p):
        self.lengthEdit.setEnabled(p)

    @pyqtSlot()
    def on_lengthEdit_validated(self):
        i = self.lengthEdit.value()
        theta = self.theta()
        c = self.center()
        l2 = QPointF(i / 2. * np.cos(theta), i / 2. * np.sin(theta))
        self.setP1((c - l2).toPoint())
        self.setP2((c + l2).toPoint())
        self.etendueEdit.move(self.etenduePos() - LigneTB.spinBoxCenter)
        self.stretchable = False
        self.parent.update()

    @pyqtSlot()
    def on_etendueEdit_validated(self):
        self.etendue=self.etendueEdit.value()
        self.parent.update()

    def allCoords(self):
        i = np.vstack(np.linspace(-self.etendue / 2., self.etendue / 2., self.etendue))
        diff = self.p1() - self.p2()
        if diff.x() == 0:
            theta = np.pi / 2
        else:
            theta = 1. * np.arctan(1. * diff.y() / diff.x())
        xx = np.linspace(self.p1().x(), self.p2().x(),
                            self.length()) - i * np.sin(theta)
        yy = np.linspace(self.p1().y(), self.p2().y(),
                            self.length()) + i * np.cos(theta)

        cond = np.logical_and(np.logical_and(np.logical_and(0 < np.max(xx, 0), \
                                                            np.min(xx, 0) < self.parent.width()), 0 < np.max(yy, 0)),
                              np.min(yy,0) < self.parent.height())
        xx = xx[:, cond].astype(np.int)
        yy = yy[:, cond].astype(np.int)
        return xx.clip(0, self.parent.width() - 1), yy.clip(0, self.parent.height() - 1)


class draggableLine(QWidget):
    newPos = pyqtSignal(QMouseEvent)
    newLine = pyqtSignal()
    delLine = pyqtSignal(int)
    orig = PointTB(0,0)

    def __init__(self, coords, etendue, parent=None, color=QColor(200, 200, 0, 150)):
        super(draggableLine, self).__init__(parent)
        self.setGeometry(parent.frameGeometry())
        self.lignes = dict()
        ligne1 = LigneTB(QPoint(coords[0, 0], coords[0, 1]), \
                         QPoint(coords[1, 0], coords[1, 1]), parent=self, etendue=etendue)
        self.lignes[0] = ligne1
        self.inMove = False
        self.clickPos = draggableLine.orig

    def paintEvent(self, e):
        painter = QPainter(self)
        for l in self.lignes.values():
            l.lengthEdit.show()
            l.etendueEdit.show()
            painter.setPen(QPen(l.color,2,l.lineStyle))  # self.etendue, Qt.SolidLine));
            painter.drawLine(l.p1(), l.p2())
            painter.setPen(QPen(l.color2, l.etendue,
                                Qt.SolidLine, Qt.FlatCap, Qt.BevelJoin));
            painter.drawLine(l.p1(), l.p2())
            painter.setPen(QPen(l.color, l.etendue,
                                Qt.SolidLine, Qt.RoundCap, Qt.BevelJoin))
            painter.drawPoints(l.p1(), l.p2())

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            pris = False
            for l in self.lignes.values():
                p1 = l.p1()
                p2 = l.p2()
                if p1.distanceTo(event.pos()) < l.etendue/2.:
                    l.offset = event.pos()
                    l.p1InMove = True
                    pris = True
                    continue
                elif p2.distanceTo(event.pos()) < l.etendue/2.:
                    l.offset = event.pos()
                    l.p2InMove = True
                    pris = True
                    continue
                if np.min((p1.x(), p2.x()))-l.etendue/2. < event.x() < np.max((p1.x(), p2.x()))+l.etendue/2.:
                    if np.min((p1.y(), p2.y()))-l.etendue/2. < event.y() < np.max((p1.y(), p2.y()))+l.etendue/2.:
                        diff = p1 - p2
                        if diff.x() == 0:
                            distance = np.abs(p1.x() - event.x())
                        else:
                            a = 1. * diff.y() / diff.x()
                            b = 1. * p1.y() * p2.x() - p2.y() * p1.x()
                            b /= (-1. * p1.x() + p2.x())
                            distance = np.abs((a * event.x() - event.y() + b) / np.sqrt(1 + a ** 2))
                        if distance <= l.etendue/2.:
                            l.offset = event.pos()
                            l.inMove = True
                            pris = True
                            continue
            if pris:
                return
            self.clickPos = PointTB(event.pos())
        elif event.button() == Qt.RightButton:
            for i, l in self.lignes.items():
                p1 = l.p1()
                p2 = l.p2()
                if np.min((p1.x(), p2.x()))-l.etendue/2. < event.x() < np.max((p1.x(), p2.x()))+l.etendue/2.:
                    if np.min((p1.y(), p2.y()))-l.etendue/2. < event.y() < np.max((p1.y(), p2.y()))+l.etendue/2.:
                        diff = p1 - p2
                        if diff.x() == 0:
                            distance = np.abs(p1.x() - event.x())
                        else:
                            a = 1. * diff.y() / diff.x()
                            b = 1. * p1.y() * p2.x() - p2.y() * p1.x()
                            b /= (-1. * p1.x() + p2.x())
                            distance = np.abs((a * event.x() - event.y() + b) / np.sqrt(1 + a ** 2))
                        if distance <= l.etendue/2.:
                            QObjectCleanupHandler().add(l.lengthEdit)
                            QObjectCleanupHandler().add(l.etendueEdit)
                            self.delLine.emit(i)
                            del self.lignes[i]
                            return

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton:
            for l in self.lignes.values():
                l.p1InMove= l.p2InMove = l.inMove = False
            return

    def mouseMoveEvent(self, event):
        pris = False
        if event.buttons() == Qt.LeftButton:
            for l in self.lignes.values():
                if l.inMove:
                    l.setP1(event.pos() - l.offset + l.p1())
                    l.setP2(event.pos() - l.offset + l.p2())
                    l.offset = event.pos()
                    l.lengthEdit.move(l.center() - LigneTB.spinBoxCenter)
                    l.etendueEdit.move(l.etenduePos() - LigneTB.spinBoxCenter)
                    pris = True
                    continue
                if l.p1InMove:
                    if l.stretchable:
                        l.setP1(event.pos() - l.offset + l.p1())
                        l.offset = event.pos()
                    else:
                        l.setP1(l.future_p(l.p2(),event.pos()))
                    l.lengthEdit.move(l.center() - LigneTB.spinBoxCenter)
                    l.lengthEdit.setValue(l.length())
                    l.etendueEdit.move(l.etenduePos() - LigneTB.spinBoxCenter)
                    pris = True
                    continue
                if l.p2InMove:
                    if l.stretchable:
                        l.setP2(event.pos() - l.offset + l.p2())
                        l.offset = event.pos()
                    else:
                        l.setP2(l.future_p(l.p1(),event.pos()))
                    l.offset = event.pos()
                    l.lengthEdit.move(l.center() - LigneTB.spinBoxCenter)
                    l.lengthEdit.setValue(l.length())
                    l.etendueEdit.move(l.etenduePos() - LigneTB.spinBoxCenter)
                    pris = True
                    continue
            if pris:
                self.update()
                return
            if self.clickPos != draggableLine.orig and self.clickPos.distanceTo(event.pos()) > 10:
                i=0
                while i in self.lignes:
                    i+=1
                l=LigneTB(self.clickPos, event.pos(), parent=self, nb=i)
                if len(self.lignes)>0:
                    l.etendueEdit.setValue(self.lignes[self.lignes.keys()[-1]].etendue)
                    l.etendue=self.lignes[self.lignes.keys()[-1]].etendue
                l.offset = event.pos()
                l.p2InMove = True
                self.lignes[i]=l
                self.update()
                self.clickPos = draggableLine.orig
                self.newLine.emit()
                return

    def mouseDoubleClickEvent(self, event):
        for l in self.lignes.values():
            if not l.lengthEdit.isEnabled():
                diff = event.pos() - l.lengthEdit.pos()
                if (0<diff.x()<l.lengthEdit.width()) and (0<diff.y()<l.lengthEdit.height()):
                    l.stretchable = True

    def resizeEvent(self, QResizeEvent):
        for l in self.lignes.values():
            l.lengthEdit.setMaximum(np.hypot(self.width(), self.height()))
        QWidget.resizeEvent(self, QResizeEvent)


class WebcamProfile(QMainWindow, Ui_webcamProfile):
    def __init__(self, parent=None):
        self.lastFrameTime = self.thisFrameTime = time()
        super(WebcamProfile, self).__init__(parent) # QMainWindow.__init__(self)
        Ui_webcamProfile.__init__(self)
        # Configure l'interface utilisateur.
        self.setupUi(self)
        self.lineStyle = listWrap(('-', '--', '-.', ':'))
        self.lineColors = np.array(('blue', 'green', 'red', 'black'))
        self.cameraDevice = CameraDevice(cameraId=self.comboBox_cam.currentIndex(),mirrored=True,parent=self)
        self.camW = CameraWidget(self.cameraDevice, self.viewPort)
        coords = np.array(((100, 50), (600, 400)))
        self.mplwidget.figure.patch.set_alpha(0.)
        self.mplwidget.figure.set_tight_layout({'pad': .1})
        self.mplText = self.mplwidget.axes.text(.5, .5, "", horizontalalignment='center', verticalalignment='center',
                                                transform=self.mplwidget.axes.transAxes)
        self.axe_percent = self.mplwidget.axes.twinx()
        self.axe_percent.set_ylabel("%",y=.95,labelpad=-11,rotation="horizontal")
        self.comboBox_cam.__class__ = DropDownCamera
        self.comboBox_cam.promote(self.cameraDevice)
        self.button_play_pause.setIcon(self.style().standardIcon(QStyle.SP_MediaPause))
        self.button_record.__class__ = RecordButton
        self.button_record.promote()
        self.noLine = self.onRecord = False
        self.colors = np.array((self.cbBlue.isChecked(), self.cbGreen.isChecked(), self.cbRed.isChecked(), self.cbGrey.isChecked()))
        self.mesLignes = draggableLine(coords, 20, self.viewPort)
        self.connect_signals()
        self.mesPlots = np.zeros((4,4),dtype=matplotlib.lines.Line2D)
        self.add_plot()
        self.recordedFrames = self.nbScale = self.xlim = self.ymax = 0
        self.ymin = 255


    @pyqtSlot(np.ndarray)
    def on_new_frame(self, frame):
        self.thisFrameTime=time()
        self.label_fps.setText(str(round(1./(self.thisFrameTime-self.lastFrameTime),1)))
        self.lastFrameTime = self.thisFrameTime

        if self.onRecord:
            self.recordedFrames += 1
            self.button_record.remaining_frames = 10-self.recordedFrames
            self.button_record.update()
        self.colors = np.array((self.cbBlue.isChecked(), self.cbGreen.isChecked(), self.cbRed.isChecked(), self.cbGrey.isChecked()))
        # self.label_expT.setText(str(self.cameraDevice.exposureTime))
        if self.colors[0:3].any():
            if self.onRecord:
                self.frame4c[:, :, 0:3] += frame[:,:,::-1]
            else:
                self.frame4c[:, :, 0:3] = frame[:,:,::-1]
        else:
            if not self.cbGrey.isChecked():
                return
        if self.cbGrey.isChecked():
            if self.onRecord:
                self.frame4c[:, :, 3] += cvtColor(frame, COLOR_RGB2GRAY)
            else:
                self.frame4c[:, :, 3] = cvtColor(frame, COLOR_RGB2GRAY)

        if self.recordedFrames == 9:
            self.frame4c /= 1.* (self.recordedFrames +1)
            self.toggle_record()
            self.pause()
            self.camW._frame= np.uint8(self.frame4c[:, :, 2::-1])
        self.xlim = 0
        if self.cbAutoscale.isChecked():
            self.nbScale = self.ymax = 0
            self.ymin = 255
        else:
            self.nbScale += 1
        for i,l in self.mesLignes.lignes.items():
            xx, yy = l.allCoords()
            wideProfile = 1. * self.frame4c[:,:,self.colors][yy, xx].sum(0) / l.etendue
            wideProfile = gaussian_filter1d(wideProfile.T,l.etendue).T
            self.xlim = max(self.xlim, l.length())
            lmax = np.max(wideProfile)
            lmin = np.min(wideProfile)
            if lmax > self.ymax:
                if self.cbAutoscale.isChecked():
                    self.ymax= lmax
                elif self.nbScale < 10:
                    self.ymax = min(255,1.1*lmax)
            if lmin < self.ymin:
                if self.cbAutoscale.isChecked():
                    self.ymin = lmin
                elif self.nbScale < 10:
                    self.ymin = max(0,.9*lmin)
            for j in range(wideProfile.shape[1]):
                self.mesPlots[i,self.colors][j].set_data(np.arange(wideProfile.shape[0]),wideProfile[:, j])

        if len(self.mesLignes.lignes)>0:
            self.mplwidget.axes.set_xlim(0, self.xlim)
            if self.cbAutoscale.isChecked():
                self.mplwidget.axes.set_ylim(self.ymin, self.ymax)
                self.axe_percent.set_ylim(100.*self.ymin/self.ymax, 100)
            elif self.nbScale < 10:
                self.mplwidget.axes.set_ylim(self.ymin, self.ymax)
                self.axe_percent.set_ylim(100.*self.ymin/self.ymax, 100)
            self.mplwidget.draw()
        elif not self.noLine:
            self.noLine = True
            self.mplText=self.mplwidget.axes.text(.5, .5, "draw a line on the webcam image\n to get a profile",
                                                  horizontalalignment='center', verticalalignment='center',
                                                  transform=self.mplwidget.axes.transAxes)
            self.mplwidget.draw()

        if self.recordedFrames == 9:
            self.recordedFrames = 0
            self.button_record.remaining_frames=10
            filename = QFileDialog.getSaveFileName(self,"Save result as","screenshot.png","Images (*.png)")
            QPixmap.grabWindow(self.winId()).save(filename, "png")

    def add_plot(self):
        if self.noLine:
            self.noLine = False
            if not self.cbAutoscale.isChecked():
                self.nbScale = 0
        self.colors = np.array((self.cbBlue.isChecked(), self.cbGreen.isChecked(), self.cbRed.isChecked(),
                                self.cbGrey.isChecked()))
        if len(self.mesLignes.lignes)>self.mesPlots.shape[0]:
            self.mesPlots.resize((self.mesPlots.shape[0]+4,4))
        i=0
        for l in self.mesLignes.lignes:
            # xdata = np.arange(l.length())
            # ydata = np.zeros(xdata.shape[0])
            for j in np.arange(4):
                self.mplwidget.axes.hold(i > 0 or j > 0)
                self.mesPlots[i, j], = self.mplwidget.axes.plot([], [], ls=self.lineStyle[i], color=self.lineColors[j])
            i += 1

    def del_plot(self, num):
        for j in np.arange(4):
            self.mesPlots[num, j].set_data([], [])

    def play_pause(self, b):
        if self.cameraDevice.paused:
            self.un_pause()
        else:
            self.pause()

    def un_pause(self, i=None):
        self.button_play_pause.setIcon(self.style().standardIcon(QStyle.SP_MediaPause))
        self.cameraDevice.paused = False

    def pause(self):
        self.button_play_pause.setIcon(self.style().standardIcon(QStyle.SP_MediaPlay))
        self.cameraDevice.paused = True

    def color_change(self):
        self.colors = np.array((self.cbBlue.isChecked(), self.cbGreen.isChecked(), self.cbRed.isChecked(),
                                self.cbGrey.isChecked()))
        for j in np.arange(4)[self.colors == 0]:
            for i in range(len(self.mesLignes.lignes)):
                self.mesPlots[i, j].set_data([], [])
        if not self.colors.any():
            self.mplText=self.mplwidget.axes.text(.5,.5,"""Check a 'color line' checkbox to get a profile
                (they are right under the webcam image)""",
                                                  horizontalalignment='center', verticalalignment='center',
                                                  transform=self.mplwidget.axes.transAxes)
            print self.mplText
            self.mplwidget.draw()
        else:
            self.mplText.set_text('')
            self.mplwidget.draw()

    def change_cam(self, i):
        self.cameraDevice = CameraDevice(i,True)

    def change_resolution(self, i):
        resolution = str(self.comboBox_resolution.itemText(i))
        w, h = np.array(resolution.split(" ")[0].split("x"),dtype=int)
        self.cameraDevice.frameSize = [w,h]
        self.frame4c=np.zeros((h,w,4))
        if w>640:
            self.showMaximized()
        self.viewPort.setMinimumSize(QSize(w,h))
        self.camW.resize(QSize(w,h))
        self.mesLignes.resize(QSize(w,h))

    @pyqtSlot()
    def change_exposure(self, expT):
        self.cameraDevice.exposureTime = -1 * expT

    @pyqtSlot()
    def toggle_record(self):
        self.onRecord = not(self.onRecord)
        self.button_record.setChecked(self.onRecord)

    def connect_signals(self):
        self.camW.newFrame.connect(self.on_new_frame)
        self.mesLignes.newLine.connect(self.add_plot)
        self.mesLignes.delLine.connect(self.del_plot)
        self.comboBox_cam.activated.connect(self.un_pause)
        self.comboBox_cam.currentIndexChanged.connect(self.change_cam)
        self.comboBox_resolution.currentIndexChanged.connect(self.change_resolution)
        self.change_resolution(self.comboBox_resolution.currentIndex())
        self.button_play_pause.toggled.connect(self.play_pause)
        self.button_record.clicked.connect(self.toggle_record)
        self.cbRed.stateChanged.connect(self.color_change)
        self.cbGreen.stateChanged.connect(self.color_change)
        self.cbBlue.stateChanged.connect(self.color_change)
        self.cbGrey.stateChanged.connect(self.color_change)


    def write_pos_settings(self):
        qsettings = QSettings("Thierry BOTTARO", "webcamProfile");
        qsettings.setValue( "geometry", self.saveGeometry() );
        qsettings.setValue( "savestate", self.saveState() );
        qsettings.setValue( "maximized", self.isMaximized() );
        if not self.isMaximized():
            qsettings.setValue("pos", self.pos() );
            qsettings.setValue("size", self.size() );
        for widg in self.findChildren(QCheckBox):
            qsettings.setValue(widg.objectName(),widg.isChecked())
        for widg in self.findChildren(QComboBox):
            qsettings.setValue(widg.objectName(),widg.currentIndex())

    def read_pos_settings(self):
        qsettings = QSettings( "Thierry BOTTARO", "webcamProfile" );
        self.restoreGeometry(qsettings.value("geometry").toByteArray())
        self.restoreState(qsettings.value("savestate").toByteArray())
        self.move(qsettings.value("pos").toPoint());
        self.resize(qsettings.value("size").toSize());
        if qsettings.value("maximized").toBool() & (not self.isMaximized()):
            self.showMaximized();
        for widg in self.findChildren(QCheckBox):
            widg.setChecked(qsettings.value(widg.objectName()).toBool())
        for widg in self.findChildren(QComboBox):
            widg.setCurrentIndex(qsettings.value(widg.objectName()).toInt()[0])

    def closeEvent(self, event ):
        self.write_pos_settings()
        event.accept()


if __name__ == "__main__":
    app = QApplication(argv)
    app.setQuitOnLastWindowClosed(True);
    window = WebcamProfile()
    window.read_pos_settings()
    # window.move(QPoint(300, 0))
    window.show()

    appRet = app.exec_()
    import sip

    sip.delete(window)
    exit(appRet)